"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchify = void 0;
const chokidar_1 = require("chokidar");
const path_1 = require("path");
const picomatch = require("picomatch");
const through = require("through2");
function watchify(b, options = {}) {
    const cache = b._options.cache;
    const packageCache = b._options.packageCache;
    const delay = typeof options.delay === "number" ? options.delay : 100;
    const ignoreMatcher = picomatch(options.ignoreWatch || "**/node_modules/**");
    const wopts = { persistent: true };
    if (typeof options.poll === "boolean") {
        wopts.usePolling = options.poll;
    }
    else if (typeof options.poll === "number") {
        wopts.usePolling = true;
        wopts.interval = options.poll;
    }
    let pendingTimeout;
    let updating = false;
    const changingDeps = new Map();
    const fwatchers = new Map();
    const fwatcherFiles = new Map();
    const ignoredFiles = new Map();
    const collect = () => {
        b.pipeline.get("deps").push(through.obj(function (row, _, next) {
            const file = row.expose ? b._expose[row.id] : row.file;
            cache[file] = {
                source: row.source,
                deps: Object.assign({}, row.deps),
            };
            this.push(row);
            next();
        }));
    };
    const invalidate = (id) => {
        if (cache) {
            delete cache[id];
        }
        if (packageCache) {
            delete packageCache[id];
        }
        changingDeps.set(id, true);
        const watchers = fwatchers.get(id);
        if (!updating && watchers) {
            watchers.forEach((w) => {
                w.close();
            });
            fwatchers.delete(id);
            fwatcherFiles.delete(id);
        }
        // wait for the disk/editor to quiet down first:
        pendingTimeout && clearTimeout(pendingTimeout);
        pendingTimeout = setTimeout(notify, delay);
    };
    const notify = () => {
        if (updating) {
            pendingTimeout = setTimeout(notify, delay);
        }
        else {
            pendingTimeout = undefined;
            b.emit("update", changingDeps.keys());
            changingDeps.clear();
        }
    };
    const reset = () => {
        let time;
        let bytes = 0;
        b.pipeline.get("record").on("end", () => {
            time = Date.now();
        });
        b.pipeline.get("wrap").push(through(function (chunk, enc, next) {
            bytes += chunk.length;
            this.push(chunk);
            next();
        }, function () {
            const delta = Date.now() - time;
            b.emit("time", delta);
            b.emit("bytes", bytes);
            b.emit("log", `${bytes} bytes written (${(delta / 1000).toFixed(2)} seconds)`);
            this.push(null);
        }));
    };
    const watchFile = (file, dep) => {
        var _a, _b, _c;
        dep = dep || file;
        if (!ignoredFiles.has(file)) {
            ignoredFiles.set(file, ignoreMatcher(file));
        }
        if (ignoredFiles.get(file)) {
            return;
        }
        if (!fwatchers.has(file)) {
            fwatchers.set(file, []);
        }
        if (!fwatcherFiles.has(file)) {
            fwatcherFiles.set(file, new Set());
        }
        if ((_a = fwatcherFiles.get(file)) === null || _a === void 0 ? void 0 : _a.has(dep)) {
            return;
        }
        const watcher = b._watcher(dep, wopts);
        watcher.setMaxListeners(0);
        watcher.on("error", b.emit.bind(b, "error"));
        watcher.on("change", () => {
            invalidate(file);
        });
        (_b = fwatchers.get(file)) === null || _b === void 0 ? void 0 : _b.push(watcher);
        (_c = fwatcherFiles.get(file)) === null || _c === void 0 ? void 0 : _c.add(dep);
    };
    if (cache) {
        b.on("reset", collect);
        collect();
    }
    b.on("file", (file) => {
        watchFile(file);
    });
    b.on("package", (pkg) => {
        const file = path_1.join(pkg.__dirname, "package.json");
        watchFile(file);
        if (packageCache) {
            packageCache[file] = pkg;
        }
    });
    b.on("reset", reset);
    reset();
    b.on("transform", (tr, mfile) => {
        tr.on("file", (dep) => {
            watchFile(mfile, dep);
        });
    });
    b.on("bundle", (bundle) => {
        updating = true;
        const onEnd = () => {
            updating = false;
        };
        bundle.on("error", onEnd);
        bundle.on("end", onEnd);
    });
    b.close = () => {
        for (const watchers of fwatchers.values()) {
            watchers.forEach((w) => {
                w.close();
            });
        }
    };
    b._watcher = (file, opts) => {
        return chokidar_1.watch(file, opts);
    };
    return b;
}
exports.watchify = watchify;
//# sourceMappingURL=index.js.map